/********************************************************************************************
 *                                                                                          *
 * Copyright (C) 2017 Armin Felder, Dennis Beier                                            *
 * This file is part of RocketChatMobileEngine <https://git.fairkom.net/chat/fairchat>.     *
 *                                                                                          *
 * RocketChatMobileEngine is free software: you can redistribute it and/or modify           *
 * it under the terms of the GNU General Public License as published by                     *
 * the Free Software Foundation, either version 3 of the License, or                        *
 * (at your option) any later version.                                                      *
 *                                                                                          *
 * RocketChatMobileEngine is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            *
 * GNU General Public License for more details.                                             *
 *                                                                                          *
 * You should have received a copy of the GNU General Public License                        *
 * along with RocketChatMobileEngine. If not, see <http://www.gnu.org/licenses/>.           *
 *                                                                                          *
 ********************************************************************************************/



#include "channelrepository.h"
ChannelRepository::ChannelRepository( ChannelModel &channelsModel, ChannelModel &groupsModel,
                                      ChannelModel &directModel, MessagesModel &messagesModel ):
    mChannelsModel( channelsModel ), mGroupsModel( groupsModel ), mDirectModel( directModel ), mMessagesModel( messagesModel )
{
    mElements.reserve( 50 );
}

bool ChannelRepository::add( QString id, QSharedPointer<RocketChatChannel> msg )
{
    connect( msg.data(), &RocketChatChannel::messageAdded, this, &ChannelRepository::receiveTimestamp, Qt::UniqueConnection );
    QString type = msg->getType();

    if ( type == "c" ) {
        QMetaObject::invokeMethod( &mChannelsModel,  "addChannelSlot", Q_ARG( QSharedPointer<RocketChatChannel>, msg ) );
    } else if ( type == "d" ) {
        QMetaObject::invokeMethod( &mDirectModel, "addChannelSlot", Q_ARG( QSharedPointer<RocketChatChannel>, msg ) );
    } else if ( type == "p" ) {
        QMetaObject::invokeMethod( &mGroupsModel,  "addChannelSlot", Q_ARG( QSharedPointer<RocketChatChannel>, msg ) );
    }

    mNameIndex[msg->getName()] = msg;
    QMetaObject::invokeMethod( &mMessagesModel, "addChannelSlot", Q_ARG( QSharedPointer<RocketChatChannel>, msg ) );
    return AbstractBaseRepository::add( id, msg );
}

bool ChannelRepository::add( QSharedPointer<RocketChatChannel> pChannel )
{
    QString id = pChannel->getRoomId();
    mNameIndex[pChannel->getName()] = pChannel;
    return add( id, pChannel );
}

ChannelRepository::~ChannelRepository()
{

}

void ChannelRepository::receiveTimestamp( QString pId, qint64 pTimestamp )
{
    QSharedPointer<RocketChatChannel> channel = get( pId );

    if ( mChannelTimeIndex.contains( channel ) ) {
        mTimestampIndex.remove( mChannelTimeIndex[channel] );
        mChannelTimeIndex[channel] = pTimestamp;
    } else {
        mChannelTimeIndex.insert( channel, pTimestamp );
    }

    mTimestampIndex.insert( pTimestamp, channel );
    //TODO: depricated?
    emit orderChanged( pId );
}

QSharedPointer<RocketChatChannel> ChannelRepository::getChannelByName( QString pName )
{
    if ( mNameIndex.contains( pName ) ) {
        return mNameIndex[pName];
    } else {
        return nullptr;
    }
}

unsigned long ChannelRepository::getYoungestMessageDate()
{
    long oldest = LONG_MAX;


    for ( const auto &channel : mElements ) {
        auto longest = channel->getOldestMessage();

        if ( Q_LIKELY( !longest.isNull() ) ) {
            long temp = longest->getTimestamp();

            if ( temp < oldest ) {
                oldest = temp;
            }
        }
    }

    if ( oldest == LONG_MAX ) {
        oldest = 0;
    }


    return oldest;
}

unsigned long ChannelRepository::getOldestMessageDate()
{
    long youngest = 0;

    for ( const auto &channel : mElements ) {
        if ( !channel.isNull() ) {
            long temp = channel->getYoungestMessage()->getTimestamp();

            if ( temp > youngest ) {
                youngest = temp;
            }
        }
    }

    return youngest;
}

