#ifndef WINRTWEBSOCKET_H
#define WINRTWEBSOCKET_H

#include <QObject>
#include <QTimer>
#include <QVector>
#include <vccorlib.h>
#include <collection.h>
#include <ppltasks.h>
#include <windows.networking.h>
#include <windows.networking.sockets.h>
#include <windows.storage.streams.h>
#include <windows.foundation.h>
#include <windows.ui.core.h>
#include <codecvt>

#include "websocketabstract.h"
#include "WinRt/winrtfix.h"


using namespace Windows::Networking::Sockets;
using namespace Windows::Storage::Streams;
using namespace Windows::Foundation;
using namespace Platform;
using namespace Windows::Web;
using namespace Windows::UI::Core;
using namespace concurrency;


//singelton to communicate between ref class an QT
class WinRTQTSignalHelperClass:public QObject{
    Q_OBJECT
public:
    WinRTQTSignalHelperClass(){};
    WinRTQTSignalHelperClass(const WinRTQTSignalHelperClass&){};

    ~WinRTQTSignalHelperClass(){};

    static WinRTQTSignalHelperClass *helper;
    static WinRTQTSignalHelperClass* instance();
    void emitOnClosed();
    void emitOnMessageReceived(QString);
	void emitOnConnected();

signals:

    void onMessageReceived(QString);
    void onClosed();
	void onConnected();
};


//class to handle responses
ref class ReceiveContext sealed{
private:
    //ugly hack because of Microsoft...
    WinRTQTSignalHelperClass* helper;

    MessageWebSocket^ messageWebSocket;
    MessageWebSocket^ webSocket;

	bool connected = false;

    DataWriter^ messageWriter;
    void HandleException(Exception^ exception);
    void checkMessage(Platform::String^);


public:
    friend class WinRTWebsocket;
    ReceiveContext();
    void SendMessage(Platform::String^);
    void init( Uri^);
    void close();
    void OnReceive(MessageWebSocket^ sender, MessageWebSocketMessageReceivedEventArgs^ args);
    void Closed(IWebSocket^ sender, WebSocketClosedEventArgs^ args);

};

class WinRTWebsocket:public WebsocketAbstract
{
    Q_OBJECT
public:
    WinRTWebsocket();

    virtual void open(QUrl) override;
    virtual void close() override;
    virtual void sendTextMessage(QString) override;
    virtual bool isValid();

    static Platform::String^ QStringToPlatformString(QString string){
        std::string stdString = string.toStdString();
        std::wstring_convert<std::codecvt_utf8<wchar_t>> converter;
        std::wstring wstdString = converter.from_bytes(stdString);
        return ref new Platform::String(wstdString.c_str());
    }
    static QString PlatformStringToQString(Platform::String ^string){
        std::wstring wString = string->Data();

        using convert_type = std::codecvt_utf8<wchar_t>;
        std::wstring_convert<convert_type, wchar_t> converter;
        std::string stdString = converter.to_bytes(wString);

        QString retString = QString::fromStdString(stdString);

        return retString.length()?retString:"";
    }

    ~WinRTWebsocket();

private:
    QTimer *timer;
    bool connectionOpen=false;

    Uri^ serverAdrr;
    int newMessage=0;
    WinRTQTSignalHelperClass* helper;

    ReceiveContext^ m_context;

    void checkForConnection();
signals:
   void  MessageReceived(QString);

private slots:
    void emitOnMessageReceived(QString);
    void emitOnClosed();
};



#endif // WINRTWEBSOCKET_H
